<?php
// Author: Kevin K.M. Chiu
// Copyright 2000, Cobalt Networks.  All rights reserved.
// $Id: Collator.php,v 1.1 2000/05/27 03:57:58 kevin Exp $

global $isCollatorDefined;
if($isCollatorDefined)
  return;
$isCollatorDefined = true;

class Collator {
  //
  // public methods
  //

  // description: collate 2 numeric values
  // param: valueA: the first value
  // param: valueB: the second value
  // param: collator: a Collator object
  // returns: ">" if A > B, "<" if A < B, "=" if A == B
  function collateNumbers($valueA, $valueB, $collator) {
    if($valueA > $valueB)
      return ">";
    else if($valueA < $valueB)
      return "<";
    else if($valueA == $valueB)
      return "=";
  }

  // description: collate 2 string values
  // param: valueA: the first value
  // param: valueB: the second value
  // param: collator: a Collator object
  // returns: ">" if A > B, "<" if A < B, "=" if A == B
  function collateStrings($valueA, $valueB, $collator) {
    if($valueA > $valueB)
      return ">";
    else if($valueA < $valueB)
      return "<";
    else if($valueA == $valueB)
      return "=";
  }

  // description: sort an array of keys and values
  // param: keys: an array of Collatable objects
  //     sorted when the method returns
  // param: values: an array of values of any type. Optional
  //     sorted when the method returns
  // returns: none
  function sort(&$keys, &$values) {
    // this method makes use of quicksort

    // no sorting needed?
    if(count($keys) == 0)
      return $keys;

    // got values?
    $isValues = true;
    if($values != "" && count($values) == 0)
      $isValues = false;

    // start with the whole array
    $jobs = array("0:".(count($keys)-1));

    // finish all jobs
    while(count($jobs) > 0) {
      // get the job
      $job = array_pop($jobs);

      // find the start and the end
      $startEnd = explode(":", $job);
      $startIndex = $startEnd[0];
      $endIndex = $startEnd[1];

      // let pivot be the end
      $pivotIndex = $endIndex;
      $pivotValue = $keys[$pivotIndex]->getCollatableValue();

      // index of the greater value
      $greaterIndex = $startIndex;
      // index of the lesser value
      $lesserIndex = $endIndex;

      // partition
      while($greaterIndex < $lesserIndex) {
	// find the first element greater than pivot from the start
	while($greaterIndex < $endIndex) {
	  $greaterValue = $keys[$greaterIndex]->getCollatableValue();

	  $result = $keys[$pivotIndex]->collate($greaterValue, $pivotValue, $this);
	  if($result == ">" || $result == "=")
	    break;

	  $greaterIndex++;
	}

        // find the first element lesser than pivot from the end
        while($lesserIndex > $startIndex) {
	  $lesserValue = $keys[$lesserIndex]->getCollatableValue();

	  if($keys[$pivotIndex]->collate($lesserValue, $pivotValue, $this) == "<")
	    break;

	  $lesserIndex--;
	}

	// swap
	if($greaterIndex < $lesserIndex) {
	  // keys
	  $tmp = $keys[$greaterIndex];
	  $keys[$greaterIndex] = $keys[$lesserIndex];
	  $keys[$lesserIndex] = $tmp;

	  // values
	  if($isValues) {
	    $tmp = $values[$greaterIndex];
	    $values[$greaterIndex] = $values[$lesserIndex];
	    $values[$lesserIndex] = $tmp;
	  }
	}
      }

      // swap key with pivot
      $tmp = $keys[$greaterIndex];
      $keys[$greaterIndex] = $keys[$pivotIndex];
      $keys[$pivotIndex] = $tmp;

      // swap value with pivot
      if($isValues) {
	$tmp = $values[$greaterIndex];
	$values[$greaterIndex] = $values[$pivotIndex];
	$values[$pivotIndex] = $tmp;
      }

      // make new jobs if needed
      if($startIndex < $greaterIndex-1)
	$jobs[] = $startIndex.":".($greaterIndex-1);
      if($greaterIndex+1 < $endIndex)
	$jobs[] = ($greaterIndex+1).":".$endIndex;
    }
  }
}
