<?php
// Author: Kevin K.M. Chiu
// Copyright 2000, Cobalt Networks.  All rights reserved.
// $Id: CompositeFormField.php,v 1.10 2001/05/15 07:05:06 jcheng Exp $

// description:
// This class is a container of FormField objects.
//
// applicability:
// This class is useful for putting in multiple form fields into places that
// accept one form field object.
//
// usage:
// Simple use addFormField() to add form fields into this object, then use
// toHtml() to get a HTML representation of it.

global $isCompositeFormFieldDefined;
if($isCompositeFormFieldDefined)
  return;
$isCompositeFormFieldDefined = true;

include("uifc/FormField.php");

class CompositeFormField extends FormField {
  //
  // private variables
  //

  var $delimiter;
  var $formFields;
  var $alignment;

  //
  // public methods
  //

  // description: constructor
  function CompositeFormField() {
    $this->formFields = array();

    // set default
    $this->delimiter = " ";
  }

  function getCollatableValue() {
    return -count($this->formFields);
  }

  // description: get the delimiter to separate form fields
  // returns: a delimiter in string
  function getDelimiter() {
    return $this->delimiter;
  }

  // description: set the delimiter to separate form fields
  // param: delimiter: a delimiter in string
  function setDelimiter($delimiter) {
    $this->delimiter = $delimiter;
  }

  // description: get form fields added to this object
  // returns: an array of FormField object
  // see: addFormField()
  function getFormFields() {
    return $this->formFields;
  }

  // description: set vertical alignment of the horizontal row of form fields
  // param: "top", "middle", "bottom"
  function setAlignment($alignment) {
    $this->alignment = $alignment;
  }

  // description: add a form field to this object
  // param: formField: a FormField object
  // see: getFormFields()
  function addFormField($formField) {
    $this->formFields[] = $formField;
  }

  function toHtml($style = "") {
    if ($this->alignment) {
      $align_text = "VALIGN=" . $this->alignment;
    }

    $result .= "<TABLE BORDER=\"0\" CELLPADDING=\"0\" CELLSPACING=\"0\"><TR $align_text>";

    $delimiter .= $this->getDelimiter();
    $formFields = $this->getFormFields();
    for($i = 0; $i < count($formFields); $i++) {
      if($i > 0)
	$result .= "<TD>$delimiter</TD>";
      $result .= "<TD>".$formFields[$i]->toHtml($style)."</TD>";
    }

    $result .= "</TR></TABLE>";

    return $result;
  }
}
