<?php
// Author: Kevin K.M. Chiu
// Copyright 2000, Cobalt Networks.  All rights reserved.
// $Id: Option.php,v 1.8 2001/04/13 01:55:21 kevin Exp $

// description:
// This class represents an option for the MultiChoice class.
//
// applicability:
// Use it where MultiChoice is used.

global $isOptionDefined;
if($isOptionDefined)
  return;
$isOptionDefined = true;

class Option {
  //
  // private variables
  //

  var $formFields;
  var $formFieldLabels;
  var $label;
  var $isSelected;
  var $value;

  //
  // public methods
  //

  // description: constructor
  // param: label: a Label object
  // param: value: the value of this option
  // param: isSelected: true if selected, false otherwise
  //     Optional and false by default
  function Option(&$label, $value, $isSelected = false) {
    $this->setLabel($label);
    $this->setValue($value);
    $this->setSelected($isSelected);

    $this->formFields = array();
    $this->formFieldLabels = array();
  }

  // description: get the label
  // returns: a Label object
  // see: setLabel()
  function &getLabel() {
    return $this->label;
  }

  // description: set the label
  // param: a Label object
  // see: getLabel()
  function setLabel(&$label) {
    $this->label =& $label;
  }

  // description: see if the option is selected
  // returns: true if selected, false otherwise
  // see: setSelected()
  function isSelected() {
    return $this->isSelected;
  }

  // description: select/unselect the option
  // param: isSelected: true to select, false to unselect
  // see: isSelected()
  function setSelected($isSelected) {
    $this->isSelected = $isSelected;
  }

  // description: get the value
  // returns: a string
  // see: setValue()
  function getValue() {
    return $this->value;
  }

  // description: set the value
  // param: value: a string
  // see: getValue()
  function setValue($value) {
    $this->value = $value;
  }

  // description: get all the form fields of the block
  // returns: an array of FormField objects
  function getFormFields() {
    return $this->formFields;
  }

  // description: get the label for a form field
  // param: formField: a FormField object
  // returns: a Label object
  function &getFormFieldLabel(&$formField) {
    return $this->formFieldLabels[$formField->getId()];
  }

  // description: add a form field to this option,
  //     so this option can associate with another form field
  // param: formField: a FormField object
  // param: label: a Label object. Optional
  function addFormField(&$formField, $label = "") {
    $this->formFields[] =& $formField;
    $this->formFieldLabels[$formField->getId()] = $label;
  }
}
