<?php
// Copyright 2001, Sun Microsystems.  All rights reserved.
// $Id: PlainBlock.php,v 1.1 2001/03/05 23:56:16 ebraswell Exp $

// description:
// Plain Block represents a block with NO title and HtmlComponents in it.
//
// applicability:
// Use where you need a block of HTMLComponents without a title 
//
// usage:
// After instantiation, use addHtmlComponent() to add HTML components into the
// block. Use toHtml() to get HTML representation.

global $isPlainBlockDefined;
if($isPlainBlockDefined)
  return;
$isPlainBlockDefined = true;

include("uifc/Block.php");

class PlainBlock extends Block {
  //
  // private variables
  //

  var $components;
  var $dividers;
  var $dividerIndexes;

  //
  // public methods
  //

  // description: constructor
  // param: page: the Page object this block is in
  // param: label: a Label object for the block title. Optional
  function PlainBlock($page, $label = "") {
    // superclass constructor
    $this->Block($page, $label);

    $this->components = array();
    $this->dividers = array();
    $this->dividerIndexes = array();
  }

  function getDefaultStyle($stylist) {
    return $stylist->getStyle("PlainBlock");
  }

  // description: get all the HTML components of the block
  // returns: an array of HtmlComponent objects
  // see: addHtmlComponent()
  function getHtmlComponents() {
    return $this->components;
  }

  // description: add a HTML component to this block
  // param: htmlComponent: a HtmlComponent object
  // see: getHtmlComponents()
  function addHtmlComponent($htmlComponent) {
    $this->components[] = $htmlComponent;
  }

  // description: get all dividers added to the block
  // returns: an array of Label objects
  // see: addDivider()
  function getDividers() {
    return $this->dividers;
  }

  // description: add a divider
  // param: label: a label object. Optional
  // see: getDividers()
  function addDivider($label = "") {
    $this->dividers[] = $label;

    // find the number of HTML components before the divider on the page
    $components = $this->getHtmlComponents();

    $this->dividerIndexes[] = count($components);
  }

  function toHtml($style = "") {
    $page = $this->getPage();
    $i18n = $page->getI18n();

    if($style == null || $style->getPropertyNumber() == 0)
      $style = $this->getDefaultStyle($page->getStylist());

    // find out style properties
    $borderColor = $style->getProperty("borderColor");
    $borderThickness = $style->getProperty("borderThickness");
    $dividerAlign = $style->getProperty("dividerAlign");
    $dividerLabelStyle = $style->getSubstyle("dividerLabel");
    $dividerHeight = $style->getProperty("dividerHeight");
    $dividerStyleStr = $style->toBackgroundStyle("dividerCell");
    $htmlComponentStyleStr = $style->toBackgroundStyle("htmlComponentCell");

    $width = ($width == -1) ? $style->getProperty("width") : $this->width;

    $form = $page->getForm();
    $formId = $form->getId();


    // find all HTML components
    $components = $this->getHtmlComponents();

    // find all dividers
    $dividers = $this->getDividers();
    $dividerIndexes = $this->dividerIndexes;

    $result = "
<TABLE BGCOLOR=\"$borderColor\" BORDER=\"0\" CELLPADDING=\"0\" CELLSPACING=\"0\" WIDTH=\"$width\"><TR><TD>
<TABLE BORDER=\"0\" CELLPADDING=\"0\" CELLSPACING=\"$borderThickness\" WIDTH=\"100%\">\n";

    for($i = 0; $i < count($components); $i++) {
      // add dividers
      for($j = 0; $j < count($dividers); $j++) {
	// divider at the right position?
	if($dividerIndexes[$j] == $i) {
	  $labelObj = $dividers[$j];
	  $label = is_object($labelObj) ? $labelObj->toHtml($dividerLabelStyle) : "";
	  $result .="<TR><TD ALIGN=\"$dividerAlign\" STYLE=\"$dividerStyleStr\" HEIGHT=\"$dividerHeight\"><IMG BORDER=\"0\" SRC=\"/libImage/spaceHolder.gif\" WIDTH=\"5\">$label</TD></TR>"; 
	}
      }

      // get HTML
      $componentHtml = $components[$i]->toHtml();

      $result .= "
	<TR>
	  <TD STYLE=\"$htmlComponentStyleStr\">$componentHtml</TD>
	</TR>
";
    }

    // add last dividers
    $componentCount = count($components);
    for($i = 0; $i < count($dividers); $i++) {
      // divider at the last position?
      if($dividerIndexes[$i] >= $componentCount) {
	$label = is_object($dividers[$i]) ? $dividers[$i]->toHtml($dividerLabelStyle) : "";
	$result .="<TR><TD STYLE=\"$dividerStyleStr\" HEIGHT=\"$dividerHeight\"><IMG BORDER=\"0\" SRC=\"/libImage/spaceHolder.gif\" WIDTH=\"5\">$label</TD></TR>"; 
      }
    }

    // make buttons
    $buttons = $this->getButtons();
    if(count($buttons) > 0) {
      $allButtons .= "<BR>";
      $allButtons .= "<TABLE BORDER=\"0\" CELLPADDING=\"0\" CELLSPACING=\"0\"><TR>";

      for($i = 0; $i < count($buttons); $i++) {
	if($i > 0)
	  $allButtons .=  "<TD><IMG SRC=\"/libImage/spaceHolder.gif\" WIDTH=\"5\" HEIGHT=\"5\"></TD>";
	$allButtons .= "<TD>".$buttons[$i]->toHtml()."</TD>";
      }

      $allButtons .= "</TR></TABLE>";
    }

    $result .= "</TABLE></TD></TR></TABLE>\n$allButtons\n";

    return $result;
  }
}

